import subprocess
import keras
import re
import numpy as np
import tensorflow as tf

from src.config import config
from src.ctc import ctc_decode
from os import listdir, path, system
# from wand.image import Image
# from wand.color import Color



def split_data(images, labels, shuffle=True):
    size = len(images)
    indices = np.arange(size)

    if shuffle:
        indices = keras.random.shuffle(indices)

    train_samples = int(size * (100 - config.TESTING_PERCENT) /100)
    print("Number of train samples:",train_samples)

    x_train, y_train = images[indices[:train_samples]], labels[indices[:train_samples]]
    x_valid, y_valid = images[indices[train_samples:]], labels[indices[train_samples:]]

    return x_train, y_train, x_valid, y_valid

def decode_batch_predictions(pred, max_length, num_to_char):
    input_len = np.ones(pred.shape[0]) * pred.shape[1]
    # Use greedy search. For complex tasks, you can use beam search
    results = ctc_decode(pred, input_length=input_len, greedy=True)[0][0][
        :, :max_length
    ]
    # Iterate over the results and get back the text
    output_text = []
    for res in results:
        res = tf.strings.reduce_join(num_to_char(res)).numpy().decode("utf-8")
        output_text.append(res)
    return output_text

def preprocess_dataset():
    pattern = re.compile(r"^\w{32}_[\w\d]{6}.jpeg$")

    for file in listdir(config.DATASET_PATH):
        if re.match(pattern, file):
            if not path.exists(f"{config.DATASET_PATH}/noiseless-{file}"):
                command = f"magick \"{config.DATASET_PATH}/{file}\" -threshold 50% -define connected-components:area-threshold=5 -define connected-components:mean-color=true -connected-components 4 \"{config.DATASET_PATH}/noiseless-{file}\""
                system(command)

                # For some reason, this generates pure black images. Dunno why so just using a command here /shrug
                # with Image(filename=f"{config.DATASET_PATH}/{file}") as img:
                    # img.threshold(0.3 * img.quantum_range)
                    # img.connected_components(method='4-connected', area_threshold=4, mean_color=True)
                    # img.save(filename=f"{config.DATASET_PATH}/noiseless-{file}")
